package com.sun.jmx.remote.internal;

import java.util.AbstractList;

public class ArrayQueue<T> extends AbstractList<T>
{
    private int capacity;
    private T[] queue;
    private int head;
    private int tail;
    
    public ArrayQueue(int n) {
        this.capacity = n + 1;
        this.queue = this.newArray(n + 1);
        this.head = 0;
        this.tail = 0;
    }
    
    public void resize(int capacity) {
        int size = this.size();
        if (capacity < size) {
            throw new IndexOutOfBoundsException("Resizing would lose data");
        }
        if (++capacity == this.capacity) {
            return;
        }
        Object[] array = this.newArray(capacity);
        for (int i = 0; i < size; ++i) {
            array[i] = this.get(i);
        }
        this.capacity = capacity;
        this.queue = array;
        this.head = 0;
        this.tail = size;
        return;
    }
    
    private T[] newArray(int n) {
        return (Object[])new Object[n];
    }
    
    @Override
    public boolean add(T t) {
        this.queue[this.tail] = t;
        int tail = (this.tail + 1) % this.capacity;
        if (tail == this.head) {
            throw new IndexOutOfBoundsException("Queue full");
        }
        this.tail = tail;
        return true;
    }
    
    @Override
    public T remove(int n) {
        if (n != 0) {
            throw new IllegalArgumentException("Can only remove head of queue");
        }
        if (this.head == this.tail) {
            throw new IndexOutOfBoundsException("Queue empty");
        }
        Object o = this.queue[this.head];
        this.queue[this.head] = null;
        this.head = (this.head + 1) % this.capacity;
        return o;
    }
    
    @Override
    public T get(int i) {
        int size = this.size();
        if (i < 0 || i >= size) {
            throw new IndexOutOfBoundsException("Index " + i + ", queue size " + size);
        }
        return this.queue[(this.head + i) % this.capacity];
    }
    
    @Override
    public int size() {
        int n = this.tail - this.head;
        if (n < 0) {
            n += this.capacity;
        }
        return n;
    }
}
